function [f,ce,g,ae,hl,indic] = chs (indic,xy,lambda)

% [f,ce,g,ae,hl,indic] = chs (fout,indic,xy,lambda)
%
% Simulateur pour le probleme de l'equilibre d'une chaine de barres rigides.
%
% En entree
%   fout: canal de sortie pour les impressions.
%   indic: pilote le comportement du simulateur
%    -2: analyse de la solutiona (nargin == 2, nargout == 0)
%     1: impression graphique (nargin == 2, nargout == 1)
%     2: calcul de f et ce (nargin == 2, nargout == 4)
%     4: calcul de f, ce, g et ae (nargin == 2, nargout == 7)
%     5: chs calcule hl (nargin == 3, nargout == 2).
%   xy: vecteur donnant les abscisses puis les ordonnees des noeuds.
%   lambda: multiplicateur de lagrange.
%
% En sortie
%   f: valeur de l'energie potentielle en xy,
%   ce: valeur en xy des contraintes sur la longueur des barres,
%   c1: valeur en xy des contraintes d'inegalite,
%   g: gradient de f en xy,
%   ae: jacobienne des contraintes en xy (dimension size(ce) x size(xy)),
%   hl: hessien du lagrangien en xy,
%   indic: decrit le resultat de la simulation
%    -1: variable hors domaine,
%     0: sortie normale (ce qui a ete demande a ete fait),
%     1: parametres d'entree non corrects.

% warning off all;

% Variables globales

  global LB          % longueurs desirees des barres
  global GCOL0       % caractere donnant la couleur utilisee dans les graphes
  global GCOL        % caractere donnant la couleur utilisee dans les graphes
  global GTYP0       % caractere donnant le type de ligne dans les graphes
  global GTYP        % caractere donnant le type de ligne dans les graphes
  global A B         % coordonnees du dernier noeud fixe
  global PHI PHIP    % description du sol
  global X0          % position de depart
  global FIG_SIMUL   % no de la figure du simulateur
  global FIG_SET     % 1 si la figure est definie
  global FIG_TITLE FIG_INIT FIG_ITER FIG_FOUND
  global FIG_XMIN FIG_XMAX FIG_YMIN FIG_YMAX
  global INEG        % 0: uniquement egalites, 1: aussi inegalites
  global XYFILE
  global ITER
  global FILM_FLAG   % 0: do nothing, 1: make a film, 2: save the film
  global FILM        % the film obtained from getframe
  global fout

% Initialisation

  f  = [];
  ce = [];
  g  = [];
  ae = [];
  hl = [];

% Separation des coordonnees

  n=size(xy,1);         % dimension du probleme
  nn=round(n/2);        % nombre de noeuds
  if nn*2 ~= n,
    fprintf(fout,'\nchs: >>> le nombre de variables (=%i) doit etre pair\n\n',n);
    indic=1;
    return
  end

  nb=nn+1;              % nombre de barres
  if nb < 1,
    fprintf(fout,'\nchs: >>> le nombre de barres (=%i) doit etre >= 1\n\n',nn-1);
    indic=1;
    return
  end
  me = nb;		% nbr de contraintes d'egalite

  x=xy(1:nn);
  xx=[x;A];
  xm=[0;x];
  y=xy(nn+1:n);
  yy=[y;B];
  ym=[0;y];

  LBn=LB(1:nn);
  LBp=LB(2:nb);

%-----------------------------------------------------------------------
% plot
%-----------------------------------------------------------------------

  if indic == 1,

    ITER = ITER+1;

    % graphe

    bar_width   = 1.0;
    bar_width1  = 2.0;
    point_width = 1.0;
    floor_width = 2.0;

    figure(FIG_SIMUL);
    if ~FIG_SET
      axis([FIG_XMIN FIG_XMAX FIG_YMIN FIG_YMAX]);        % pour le BGLS
      axis equal;                                         % pour le BGLS
%     axis off;
      hold on;
    end

    % la chaine initiale

    if FIG_INIT
      plot([0; X0(1:nn); A],[0; X0(nn+1:n); B],[GCOL0 GTYP0],'LineWidth',bar_width);
      plot(X0(1:nn),X0(nn+1:n),[GCOL0 '.'],'MarkerSize',20);
      plot(0,0,'ko','LineWidth',bar_width);
      plot(A,B,'ko','LineWidth',bar_width);
    end
    if FIG_TITLE
%     title('La chainette');
    end

    % la chaine trouvee

    if FIG_ITER
      if FIG_FOUND
        plot([0; x; A],[0; y; B],[GCOL GTYP],'LineWidth',bar_width1);
      else
        plot([0; x; A],[0; y; B],[GCOL GTYP],'LineWidth',bar_width);
      end
      h = plot(x,y,[GCOL '.']);
        set(h,'MarkerSize',25)
%       set(h,'MarkerSize',6.5)
        set(h,'LineWidth',bar_width)
      h = plot(0,0,'k.');
        set(h,'MarkerSize',25)
      h = plot(A,B,'k.');
        set(h,'MarkerSize',25)
      drawnow

%     if isempty(ITER); ITER = 0; end

      fprintf(XYFILE,'\nxx(:,%0i) = [',ITER);
      fprintf(XYFILE,'0');
      for i=1:nn, fprintf(XYFILE,'  %12.5e',x(i)); end
      fprintf(XYFILE,' %12.5e',A);
      fprintf(XYFILE,'];');

      fprintf(XYFILE,'\nyy(:,%0i) = [',ITER);
      fprintf(XYFILE,'0');
      for i=1:nn, fprintf(XYFILE,'  %12.5e',y(i)); end
      fprintf(XYFILE,'  %12.5e',B);
      fprintf(XYFILE,'];\n');
    end

    if FILM_FLAG == 1
%     if (ITER == 1) print -depsc FILM0.ps; end
      FILM(ITER) = getframe(gcf);		% on récupère l'image
    elseif FILM_FLAG == 2
      print -depsc FILM0.ps;
      movie2avi(FILM,'film.avi','fps',2);
    end
    indic=0;
    return

  end

%-------------------------------------------------------------------------------
% simulation
%-------------------------------------------------------------------------------

  % calcul des longueurs courantes des barres

    l = sqrt((xx-xm).^2+(yy-ym).^2);

  % calcul de f, ce, g et ae

  if indic == 2 | indic == 4,

    % f

    f = sum(LB.*(yy+ym))/2;

    % ce

    ce = l.^2-LB.^2;

    % calcul des derivees g et ae

    if indic == 4,

      % g

      g = [zeros(nn,1); (LBn+LBp)/2];

      % ae

      i = [[1:nn]';[2:nb]';[1:nn]';[2:nb]'];
      j = [[1:nn]';[1:nn]';[nb:n]';[nb:n]'];
      s = [2*(x-xm(1:nn));2*(x-xx(2:nb));
           2*(y-ym(1:nn));2*(y-yy(2:nb))];
      ae = sparse(i,j,s,me,n);

    end

  elseif indic == 5

    if nargin <= 2
      fprintf(fout,'\nchs: >>> la valeur de lambda manque en entree\n\n');
      indic = 1;
      return
    end

    i = [1:nn-1]';
    j = [2:nn]';
    s = -2*lambda(2:nn);
    hl = sparse(i,j,s,nn,nn);
    hl = hl' + 2*sparse(diag(lambda(1:nn)+lambda(2:nb))) + hl;
    hl = [   hl     zeros(nn);
          zeros(nn)    hl    ];

  else

    fprintf(fout,'\nchs: >>> valeur de indic (=%i) non implementee\n\n',indic);
    indic=1;
    return

  end

  indic=0;
